#include "accel_header.h"
#include "draw_objects.h"
#include "constants.h"

/*************************************************************************************************
* Draw Accel Screen function
*************************************************************************************************/
void DrawAccelScr()
{
  TFT_Fill_Screen(CL_BG);
  DrawLabel(stepsFig);
  DrawLabel(counterFig);
  DrawLabel(timeFig);
  DrawImage(startFig);
}
/*************************************************************************************************
* DrawImage function
* -------------------------------------------------------------------------------------------------
* Overview: Draws given image to the screen
* Input: pointer to label struct
* Output: Nothing
*************************************************************************************************/
inline void DrawImage(image_t* const imagePointer)
{
  TFT_Image(imagePointer->Left, imagePointer->top, imagePointer->Picture_Name, 1);
}
/*************************************************************************************************
* DrawLabel function
* -------------------------------------------------------------------------------------------------
* Overview: Draws given label to the screen
* Input: pointer to label struct
* Output: Nothing
*************************************************************************************************/
inline void DrawLabel(label_t* labelPointer)
{
  TFT_Set_Font(labelPointer->FontName, labelPointer->Font_Color, FO_HORIZONTAL);
  TFT_Write_Text(labelPointer->Caption, labelPointer->Left, labelPointer->Top);
}
/*************************************************************************************************
* EraseImage function
* -------------------------------------------------------------------------------------------------
* Overview: Erases given label by drawing it again in background color
* Input: pointer to label struct
* Output: Nothing
*************************************************************************************************/
inline void EraseImage(image_t* const imagePointer, uint16_t bgColor)
{
  TFT_Set_Pen(bgColor, 0);
  TFT_Set_Brush(1, bgColor, 0, 0, 0, 0);
  TFT_Rectangle(imagePointer->Left,
                imagePointer->top,
               (imagePointer->Left+imagePointer->Width),
               (imagePointer->Top + imagePointer->Height));
}
/*************************************************************************************************
* EraseLabel function
* -------------------------------------------------------------------------------------------------
* Overview: Erases given label by drawing it again in background color
* Input: pointer to label struct
* Output: Nothing
*************************************************************************************************/
inline void EraseLabel(label_t* labelPointer, uint16_t bgColor)
{
  TFT_Set_Font(labelPointer->FontName, bgColor, FO_HORIZONTAL);
  TFT_Write_Text(labelPointer->Caption, labelPointer->Left, labelPointer->Top);
}
/**************************************************************************************************
* Function InitMCU()
* -------------------------------------------------------------------------------------------------
* Overview: initialize the MCU screen
* Input: nothing
* Output: Nothing
**************************************************************************************************/
void InitMCU()
{
  GPIO_Digital_Output(&GPIOE_BASE,_GPIO_PINMASK_9);
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(SCRWIDTH, SCRHEIGHT);
  TFT_BLED = 1;
  
  // calibrate the touch-panel
  CalibrateScreen();
}
/**************************************************************************************************
* Function InitializeTouchPanel()
* -------------------------------------------------------------------------------------------------
* Overview: initialize the touch panel
* Input: nothing
* Output: Nothing
**************************************************************************************************/
void InitializeTouchPanel()
{
  Init_ADC();
  TP_TFT_Init(SCRWIDTH, SCRHEIGHT, 8, 9);                        // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADCTHRESHOLD);                       // Set touch panel ADC threshold
}
/**************************************************************************************************
* Function Init_ADC()
* -------------------------------------------------------------------------------------------------
* Overview: initialize the ADC
* Input: nothing
* Output: Nothing
**************************************************************************************************/
void Init_ADC()
{
  ADC_Set_Input_Channel(_ADC_CHANNEL_8 | _ADC_CHANNEL_9);
  ADC1_Init();
  Delay_ms(100);
}
/**************************************************************************************************
* Function Calibrate()
* -------------------------------------------------------------------------------------------------
* Overview: Function performs the calibration of the touch panel
* Input: nothing
* Output: Nothing
**************************************************************************************************/
void CalibrateScreen()
{
  InitializeTouchPanel();

  TFT_Fill_Screen(CL_WHITE);
  
  DrawImage(calibWsFig);

  TFT_Set_Pen(CL_CUSTOMRED, 3);
  TFT_Set_Font(TFT_defaultFont, CL_CUSTOMRED, FO_HORIZONTAL);
  TFT_Line(1, 319, 5, 319);
  TFT_Line(1, 319, 0, 314);
  TFT_Line(1, 319, 10, 309);
  TFT_Write_Text("first here", 10, 290);

  TP_TFT_Calibrate_Min();                      // Calibration of TP minimum
  Delay_ms(500);

  TFT_Set_Pen(CL_WHITE, 3);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Line(1, 319, 5, 319);
  TFT_Line(1, 319, 0, 314);
  TFT_Line(1, 319, 10, 309);
  TFT_Write_Text("first here", 10, 290);
  
  TFT_Set_Pen(CL_CUSTOMRED, 3);
  TFT_Set_Font(TFT_defaultFont, CL_CUSTOMRED, FO_HORIZONTAL);
  TFT_Write_Text("now here ", 160, 15);
  TFT_Line(239, 0, 234, 0);
  TFT_Line(239, 0, 239, 5);
  TFT_Line(239, 0, 229, 10);

  TP_TFT_Calibrate_Max();                       // Calibration of TP maximum
  Delay_ms(1000);
}
/**************************************************************************************************
* Function UpdateStepsCounter()
* -------------------------------------------------------------------------------------------------
* Overview: Function updates the step counter on the screen and records the current vpp
* Input: nothing
* Output: Nothing
**************************************************************************************************/
inline void UpdateStepsCounter()
{
  uint8_t i = 0;

  EraseLabel(counterFig,CL_BG);
  
  sprintf(counterCaption,"%04ld",walkStep);
  for(i = 0; i < 4*4; i++)
  {
    TFT_Set_Font(counterFig->FontName, countsColor[i/4], FO_HORIZONTAL);
    TFT_Write_Text(counterFig->Caption, counterFig->Left, counterFig->Top);
  }

  if (vppChange == 0)
  {
    vppChange = 1;
    prethVpp = vpp;
  }
}
/**************************************************************************************************
* Function UpdateScreen()
* -------------------------------------------------------------------------------------------------
* Overview: Function displays time during walking and number of steps
* Input: nothing
* Output: Nothing
**************************************************************************************************/
void UpdateScreen()
{
  if (wtSec == 60)
  {
    wtSec = 0; wtMin++;
  }
  if (wtMin == 60)
  {
    wtMin = 0;
    wtHour++;
  }
  if (updateTime == 1)
  {
    updateTime = 0;
    EraseLabel(timeFig, CL_BG);
    sprintf(timeCaption,"%02d:%02d:%02d",wtHour,wtMin,wtSec);
    DrawLabel(timeFig);
  }
  
  if (walkFlag == 1)
  {
    walkFlag = 0;
    UpdateStepsCounter(); // draw new value for the counter
  }
}
/**************************************************************************************************
* Function StartSeq()
* -------------------------------------------------------------------------------------------------
* Overview: Start sequence on the display
* Input: nothing
* Output: Nothing
**************************************************************************************************/
void StartSeq()
{
  int16_t Countdown = 5;
  EraseLabel(stepsFig, CL_BG);
  sprintf(stepsCaption,"start");
  DrawLabel(stepsFig);

  Delay_ms(500);

  countsColor = rClrz;
  
  while (Countdown > 0)
  {
    EraseLabel(counterFig,CL_BG);
    sprintf(counterCaption,"%04d",Countdown);
    for(i = 0; i < 4*4; i++)
    {
    TFT_Set_Font(counterFig->FontName, countsColor[i/4], FO_HORIZONTAL);
    TFT_Write_Text(counterFig->Caption, counterFig->Left, counterFig->Top);
    }
    Countdown--;
  }

  countsColor = yClrz;
  
  EraseLabel(stepsFig, CL_BG);
  sprintf(stepsCaption,"steps");
  DrawLabel(stepsFig);
  UpdateStepsCounter();
}