#include "built_in.h"
#include "accel_driver.h"
#include "accel_header.h"
#include "timers.h"

uint8_t dataFormat, bwRate, offsetX, offsetY, offsetZ, fifoSet, intMap, intEn, devMode;
  
/**************************************************************************************************
* Function ADXL345_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write byte of data to ADXL345
* Input: register address, data
* Output: Nothing
**************************************************************************************************/
void ADXL345_Write(uint8_t regAdr, uint8_t accelData)
{
  uint8_t accelBuffer[2];
  
  accelBuffer[0] = regAdr;
  accelBuffer[1] = accelData;
  
  Soft_I2C_Start();
  Soft_I2C_Write(_DEV_ADDR<<1 | 0);
  Soft_I2C_Write(regAdr);
  Soft_I2C_Write(accelData);
  Soft_I2C_Stop();
}

/**************************************************************************************************
* Function ADXL345_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function read byte of data from ADXL345
* Input: register address
* Output: data from addressed register in ADXL345
**************************************************************************************************/
uint8_t ADXL345_Read(uint8_t regAdr)
{
  uint8_t accelBuffer = regAdr;;
  
  Soft_I2C_Start();
  Soft_I2C_Write(_DEV_ADDR<<1 | 0);
  Soft_I2C_Write(accelBuffer);
  Soft_I2C_Start();
  Soft_I2C_Write(_DEV_ADDR<<1 | 1);
  accelBuffer = Soft_I2C_Read(0);
  Soft_I2C_Stop();
  
  return accelBuffer;
}

/**************************************************************************************************
* Function ADXL345_ReadAll()
* -------------------------------------------------------------------------------------------------
* Overview: Function read all bytes of data from ADXL345 (N = 6)
* Input: register address
* Output: data from addressed register in ADXL345
**************************************************************************************************/
void ADXL345_ReadAll(uint8_t regAdr, uint8_t* accelData) 
{

  int8_t i = 0;
  
  Soft_I2C_Start();
  Soft_I2C_Write(_DEV_ADDR<<1 | 0);
  Soft_I2C_Write(regAdr);
  Soft_I2C_Start();
  Soft_I2C_Write(_DEV_ADDR<<1 | 1);
  for(i = 0; i < NUMBYT-1; i++)
  {
    accelData[i] = Soft_I2C_Read(1);
  }
  accelData[NUMBYT-1] = Soft_I2C_Read(0);
  Soft_I2C_Stop();
}
/**************************************************************************************************
* Function ADXL345_CalibInit()
* -------------------------------------------------------------------------------------------------
* Overview: Function that initializes ADXL345 for calibrating mode
* Input: Nothing
* Output: return 0 for OK; return _ACCEL_ERROR for ERROR
**************************************************************************************************/
uint8_t ADXL345_CalibInit()
{

  // Go into standby mode to configure the device for calibration
  ADXL345_Write(_POWER_CTL, 0x00);

  if (ADXL345_Read(0x00) != 0xE5)  // device ID code
  {
    // something went wrong
    return _ACCEL_ERROR;
  }
  else // configuring the device
  {
    // disable interrupts
    ADXL345_Write(_INT_ENABLE, 0x00);
    
    // resolution, sampling rate etc.
    ADXL345_Write(_DATA_FORMAT, 0x0A);       // full resolution, +/-8g, 4mg/LSB, right justified
    ADXL345_Write(_BW_RATE, 0x09);           // 0x09 - 50Hz, 0x0A - 100 Hz

    // turn on the BYPASS mode
    ADXL345_Write(_FIFO_CTL, 0x00);

    // turn on the device
    ADXL345_Write(_POWER_CTL, 0x08);         // measurement mode ON

    // everything went well
    return 0x00;
  }
}
/**************************************************************************************************
* Function ADXL345_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function that initializes ADXL345 for streaming mode (doesn't turn it on)
* Input: Nothing
* Output: return 0 for OK; return _ACCEL_ERROR for ERROR
**************************************************************************************************/
char ADXL345_Init() {
  
  // Go into standby mode to configure the device for work
  ADXL345_Write(_POWER_CTL, 0x00);
  
  if (ADXL345_Read(0x00) != 0xE5) {     // device ID code
    // something went wrong
    return _ACCEL_ERROR;
  }
  else // configuring the device
  {
    // turn on the FIFO buffer mode with INT set @25 stored samples
    ADXL345_Write(_FIFO_CTL, 0x59);

    // everything went well
    return 0x00;
  }
}
/**************************************************************************************************
* Function AccelReadALL()
* -------------------------------------------------------------------------------------------------
* Overview: Function reads all accel values (N = 6).
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void AccelReadALL(int16_t* samples)
{
  uint8_t rawData[NUMBYT] = {0,0,0,0,0,0};

  ADXL345_ReadAll(_DATAX0, rawData); // read from _DATAX0 onwards
  
  samples[0] = ( (int16_t)rawData[1]<<8 | (int16_t)rawData[0] );
  samples[1] = ( (int16_t)rawData[3]<<8 | (int16_t)rawData[2] );
  samples[2] = ( (int16_t)rawData[5]<<8 | (int16_t)rawData[4] );
}