#include "accel_header.h"
#include "datatypes.h"
#include "dsp.h"
#include "constants.h"

/**************************************************************************************************
* Function MovAvX()
* -------------------------------------------------------------------------------------------------
* Overview: Function implements moving average fiter
* Input: signal, space for filtered signal
* Output: filtered signal
**************************************************************************************************/
void MovAvX(int32_t* sig, int32_t* filtSig, uint8_t numTap)
{
   uint8_t i;
   for(i = 0; i < NUMSAM; i++)
   {
      if ( i < (numTap-1) )
      {
         filtSig[i] = sig[i];
      }
      else
      {
         filtSig[i] = averX(sig,i,numTap);
      }
   }
}
/**************************************************************************************************
* Function AverX()
* -------------------------------------------------------------------------------------------------
* Overview: function averages a number of samples
* Input: signal, current index and a number of taps
* Output: the average value
**************************************************************************************************/
int32_t AverX(int32_t* sig, uint8_t n, uint8_t numTap)
{
   uint8_t k;
   int32_t sum = 0;
   
   for(k = 0; k < numTap; k++)
   {
        sum = sum + sig[n-k];
   }
   return sum/numTap;
}
/**************************************************************************************************
* Function FindParam()
* -------------------------------------------------------------------------------------------------
* Overview: Function calculates various parameters of the signal in buffer
* Input: array
* Output: parameters
**************************************************************************************************/
void FindParam(int32_t* array, int32_t* dynThr, int32_t* vpp, int32_t* noiseThr)
{
  uint8_t i,j;
  volatile int32_t sum = 0;
  
  memcpy((void*)sortedArray, (void*)array, NUMSAM*sizeof(int32_t));
  
  BubbleSort_t(sortedArray, NUMSAM);
  calibMax = sortedArray[NUMSAM-1];
  calibMin = sortedArray[0];
  *dynThr = (calibMin + calibMax)/2;
  *vpp = calibMax-calibMin;
  *noiseThr = (int32_t)(0.1*(*vpp));

  // free allocated memory
  Free(sortedArray,NUMSAM*sizeof(int32_t));
}

void Exchange(int32_t* number1, int32_t* number2)
{
   int32_t temp;
   temp = *number1;
   *number1 = *number2;
   *number2 = temp;
}
/**************************************************************************************************
* Function NegativeCrossing()
* -------------------------------------------------------------------------------------------------
* Overview: Function determines if the crossing of the threshold was negative
* Input: two values and a threshold
* Output: flag
**************************************************************************************************/
inline bool_t NegativeCrossing(int32_t x, int32_t y, int32_t thr)
{
   return ( ((x>=thr) && (y<thr)) || ((x>thr) && (y<=thr)) );
}
/**************************************************************************************************
* Function PositiveCrossing()
* -------------------------------------------------------------------------------------------------
* Overview: Function determines if the crossing of the threshold was positive
* Input: two values and a threshold
* Output: flag
**************************************************************************************************/
inline bool_t PositiveCrossing(int32_t x, int32_t y, int32_t thr)
{
   return ( ((x<=thr) && (y>thr)) || ((x<thr) && (y>=thr)) );
}
/**************************************************************************************************
* Function sameCross()
* -------------------------------------------------------------------------------------------------
* Overview: Function checks if two crossings are of the same type
* Input: flags for previous crossings and a present crossing
* Output: flag
**************************************************************************************************/
inline bool_t sameCross(bool_t prevCross, bool_t posCross, bool_t negCross)
{
  return ( ((prevCross == 1) && (posCross == 1)) || ((prevCross == 0) && (negCross == 1)) );
}
/**************************************************************************************************
* Function ValidStepCond()
* -------------------------------------------------------------------------------------------------
* Overview: Condition for determining if a valid step occured
* Input: various flags and a number of consecutive "steps"
* Output: flag
**************************************************************************************************/
bool_t ValidStepCond()
{
  if (currBuffer == 0)
      return 1;
  else
      return sameCross(prevCross,posCross,negCross);
}
/**************************************************************************************************
* Function findStep()
* -------------------------------------------------------------------------------------------------
* Overview: Function finds a candidate for a step
* Input: signal, current index, thresholds for noise and a dynamic threshold
* Output: flag
**************************************************************************************************/
void FindStep(int32_t* signal, uint8_t index, int32_t noiseThr, int32_t dynThr)
{
  // temporary storage
  int32_t tmp = 0;
  
  // reset the flag
  stepFlag = 0;

  // define the values to be compared
  currSam = signal[index]; // current sample
  currRef = currCand;      // current reference

  // check if the current sample value is different enough from
  // the current candidate to be even considered for a step
  if ( abs_t(currSam-currCand) >= noiseThr)
  {
    tmp = currCand; // temporary storage for the old value

    // treat the current value as a candidate for the step
    currCand = currSam;

    // has the signal went below or above the threshold?
    negCross = NegativeCrossing(currRef,currCand,dynThr);
    posCross = PositiveCrossing(currRef,currCand,dynThr);

    // crossing happened and the sample was far enough above/below the precision threshold   && (abs_t(currCand-dynThr) >= noiseThr)
    if ( Xor_t(posCross,negCross) )
    {
      // "step" happened
      stepFlag = 1;

      // increment the "step" counter
      step++;
    }
    else
    {
      currCand = tmp; // go back to the old value
    }
  }
}
/**************************************************************************************************
* Function Xor_t()
* -------------------------------------------------------------------------------------------------
* Overview: Function calculates XOR function
* Input: flags A and B
* Output: flag
**************************************************************************************************/
inline bool_t Xor_t(bool_t A, bool_t B)
{
   return ( (A && ! B) || (! A && B) );
}
/**************************************************************************************************
* Function SaveCross()
* -------------------------------------------------------------------------------------------------
* Overview: Records what kind of crossing is, for future comparisons
* Input: current sample, current threshold
* Output: flag
**************************************************************************************************/
inline bool_t SaveCross(int32_t sample, int32_t thr)
{
    if (sample > thr)
        return 1; // positive crossing
    else
        return 0; // negative crossing
}
/**************************************************************************************************
* Function abs_t()
* -------------------------------------------------------------------------------------------------
* Overview: Custom abs() function
* Input: value
* Output: absolute value
**************************************************************************************************/
int32_t abs_t(int32_t number)
{
  if (number > 0) return number;
  else if (number == 0) return 0;
  else return (-number);
}

void FindExtr(int32_t* array, int32_t size, int32_t* min, int32_t* max)
{
  BubbleSort_t(array, size);
  *min = array[0];
  *max = array[size-1];
}
/**************************************************************************************************
* Function BubbleSort_t()
* -------------------------------------------------------------------------------------------------
* Overview: sorts an array
* Input: array
* Output: sorted array
**************************************************************************************************/
void BubbleSort_t(int32_t* arr, int32_t n)
{
  int32_t i, j = 0;
  int32_t tmp;
  
  bool_t swapped = 1;

  while (swapped == 1)
  {
    swapped = 0;
    j++;
    for(i = 0; i < n - j; i++)
    {
      if (arr[i] > arr[i + 1])
      {
        tmp = arr[i];
        arr[i] = arr[i + 1];
        arr[i + 1] = tmp;
        swapped = 1;
      }
    }
  }
}