/*
 * Intern project name:
     Pedometer
 * Copyright:
     (c) mikroElektronika, 2014.
 * Revision History:
     20140904:
       - release version (ML);
 * Description:
     Pedometer - counts steps a user has made with proper usage, up to 90% precision
 * Test configuration:
     MCU:             STM32F407VG
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/DM00037051.pdf
     Dev. Board:      mikromedia for STM32 M4
                      http://www.mikroe.com/mikromedia/stm32-m4/
     Oscillator:      HS-PLL 140.0000 MHz, internal 16.0000 MHz RC
     Ext. Modules:    None.
     SW:              mikroC PRO for ARM v4.2.0
                      http://www.mikroe.com/mikroc/arm/
 */

#include "accel_header.h"
#include "timers.h"
#include "accel_driver.h"
#include "dsp.h"
#include "built_in.h"
#include "events.h"

void main(void)
{
  // initialize the screen
  InitMCU();

  // welcome screen
  DrawImage(welScrFig);
  
  // calibrate the accelerometer while standing still
  AccelCalibrate(&CALIB_status);

  // draw the figures
  DrawAccelScr();
  
  // configure the accelerometer for work
  AccelStart(&ACCEL_status);
  
  if (CALIB_status == 1)
  {
    while(1)
    {
      Check_TP();
      if (startPedometer == 1)
      {
        UpdateScreen();
        if (CheckBuffer() == 1) // read the samples
        {
            for(i = 0; i < NUMSAM; i++)
            {
              AccelReadALL(samples);
              accX[i] = (int32_t)samples[0] - calibX;
              accY[i] = (int32_t)samples[1] - calibY;
              accZ[i] = (int32_t)samples[2] - calibZ;

              accMod[i] = ( accX[i]*accX[i] +
                            accY[i]*accY[i] +
                            accZ[i]*accZ[i] );
            }
            calcSteps = 1;
        }
        else if (calcSteps == 1)
        {
          // initialize
          if (begin == 1)
          {
            begin = 0;
            currCand = accMod[0];
          }
          // calculate thresholds on moduo signal
          FindParam(accMod,&dynThr,&vpp,&noiseThr);
          
            if (vppChange == 1)
            {
              vppChange = 0;
              tmpThr = THPER*prethVpp;
              if (tmpThr > MINWALKVPP) currThr = tmpThr;
            }
            if (vpp > currThr) // if the range of the signal in buffer is big enough, there might be walking
            {
              EraseLabel(idleFig,CL_BG); EraseLabel(thrBigFig,CL_BG);

              // we're at the beginning of the buffer
              currBuffer = 0;

              for(i = 0; i < NUMSAM; i++)
              {
                if (initStep == 1) // first step
                {
                  FindStep(accMod,i,noiseThr,dynThr);

                  if (stepFlag == 1)
                  {
                  validStep++; win = 0;
                  initStep = 0;
                  }
                }
                else // if there was an initial step
                {
                  if (win < (10*WINMAX)) win++; // increase the window

                  // has this step happened in an eligible timeframe?
                  cond1 = (win > WINMIN);  // if too quick, discard noise
                  cond2 = (win <= WINMAX); // if too slow, discard individual steps
                  cond3 = (win >= (5*WINMAX)); // if nothing has happenned for 10 seconds... activity stopped?

                  if (cond1) // if the incidence wasn't too short, it may be a valid step
                  {
                    // if we just got out of forbidden zone, initialize
                    if (win == WINMIN+1)
                    {
                      if (i != 0) currCand = accMod[i-1];
                      else currCand = accMod[i];
                    }

                    FindStep(accMod,i,noiseThr,dynThr);

                    if ( (stepFlag == 1) && ValidStepCond() )
                    {
                      validStep++; win = 0;
                      // first valid step in this buffer? update the flag which controls the condition for future valid steps
                      if (currBuffer == 0) currBuffer = 1;

                      // record what kind of crossing is, for future comparisons
                      prevCross = SaveCross(currCand,dynThr);

                      if (cond2) // if the incidence wasn't too long, maybe it is walking
                      {
                        // increase the number of consecutive valid steps
                        consecValSteps++;

                        // if we are already walking, then just increase the step counter
                        if (walkingMode == 1)
                        {
                          walkStep += 1; // regular increment
                          walkFlag = 1; // mark that change has happened
                          vppChange = 1; // same purpose
                        }
                        else if (consecValSteps == 3) // no walking till now, but there were 3 consecutive valid steps?
                        {
                          walkingMode = 1; // walking registered
                          walkStep += 4; // increment the counter by 4
                          walkFlag = 1; // mark that change has happened
                          vppChange = 1; // same purpose
                        }
                      }
                      else// too much time has passed
                      {
                        consecValSteps = 0; // combo breaker
                        walkingMode = 0;    // no more walking
                      }
                    }
                  }
                }
              }
              if (vppChange == 1) prethVpp = vpp;
            }
            else if (vpp <= 2*stillVpp)  // else, don't do anything, just increase the window, if necessary
            {
              EraseLabel(thrBigFig,CL_BG);
              DrawLabel(idleFig);
              begin = 1;
              if (initStep == 0) win += NUMSAM;
            }
            else
            {
              EraseLabel(idleFig,CL_BG);  // display message that the signal is somewhere in between
              DrawLabel(thrBigFig);
              begin = 1;
              if (initStep == 0) win += NUMSAM;
            }
          calcSteps = 0;
        }
      }
    }
  }
  else  // aceelerometer hasn't been properly initialized
  {
    EraseLabel(stepsFig,CL_BG);
    EraseLabel(counterFig,CL_BG);
    sprintf(counterCaption,"ERR");
    DrawLabel(counterFig);
  }
}